#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
脚本功能：
    安装Mellanox网卡的驱动
使用示例：
    本地调用：python  install_mlnx_ofed.py mellanox_package_path=xxxx
    hcm调用：hcm psh {node} -t 1200 -f os/install_mlnx_ofed.py mellanox_package_path=xxxx

注意：该脚本执行需要依赖yum源配置OK
mellanox_package_path: Mellanox驱动包存放目录, 若为麒麟系统，tcsh.rpm依赖包需要放在同一路径
"""
import os
import subprocess
import re
import sys
import tempfile

# mellanox_package_path  （必填项）Mellanox驱动包存放目录，若为麒麟系统，tcsh.rpm包需要放在同一路径
REMOTE_PARAMETERS = " ".join(sys.argv[1:])

ARGV_EXAMPLE_DIC = {"mellanox_package_path": "xxxx"}

# 存放输入的参数值
ARGV_INPUT_DIC = ARGV_EXAMPLE_DIC


def script_params_example():
    message = "parameter example: "
    for key, value in ARGV_EXAMPLE_DIC.items():
        message = message + key + "=" + value + " "
    return message.strip()


def validate_script_params():
    if len(REMOTE_PARAMETERS) == 0:
        print('Please enter parameters, ' + script_params_example())
        exit(255)
    tmp_argv = REMOTE_PARAMETERS.split(" ")
    for item in tmp_argv:
        argv = item.split("=")
        if len(argv) != 2:
            print('Invalid parameters format, ' + script_params_example())
            exit(255)
        if argv[0] not in ARGV_EXAMPLE_DIC.keys():
            print('Parameter [ ' + argv[0] + ' ] is not defined, ' + script_params_example())
            exit(255)
        if argv[1].strip() == "" and ARGV_EXAMPLE_DIC.get(argv[0]) == "xxxx":
            print('Parameter [ ' + argv[0] + ' ] cannot be empty.')
            exit(255)
        ARGV_INPUT_DIC[argv[0]] = argv[1]

    for key, value in ARGV_INPUT_DIC.items():
        if value == "xxxx":
            print('Parameter [ ' + key + ' ] cannot be empty.')
            exit(255)

    if len(ARGV_EXAMPLE_DIC) != len(ARGV_INPUT_DIC):
        print('Number of parameters does not match, ' + script_params_example())
        exit(255)
    validate_business_params()


def validate_business_params():
    mellanox_package_path = ARGV_INPUT_DIC.get("mellanox_package_path")
    if not os.path.exists(mellanox_package_path):
        print('mellanox_package_path is not exist')
        exit(255)


# 安装mellanox驱动前校验
def pre_install_check():
    # 检查系统是否有Mellanox网卡设备
    process1 = subprocess.Popen(["lspci"], shell=False, stdout=subprocess.PIPE)
    process2 = subprocess.Popen(["grep", "Mellanox"], shell=False, stdin=process1.stdout,
        stdout=subprocess.PIPE)
    process2.communicate(timeout=5)
    ret = process2.returncode
    if ret != 0:
        print("ERROR: No Mellanox network card found.")
        exit(255)
    print("Mellanox network card found.")


# 安装mellanox网卡驱动依赖包
def install_mellanox_driver_depend(mellanox_dirver_path):
    # 获取操作系统版本
    with open('/etc/system-release', 'r') as f:
        system_version = f.read().strip()
        # 安装麒麟HPC版系统Mellanox驱动依赖包
        if 'Kylin Linux Advanced Server release V10 (Hydrogen)' in system_version:
            # 若系统内核版本为 5.10.0-153.5.v2305.ky10h.aarch64，安装依赖包不同
            kernel_version = subprocess.Popen(["uname", "-r"],
                shell=False, stdout=subprocess.PIPE).stdout.read().decode("utf-8").strip()
            if '5.10.0-153.5.v2305.ky10h.aarch64' in kernel_version:
                cmd = "yum install perl tk pciutils-devel fuse-devel tcsh make lsof".split()
            else:
                cmd = "yum install perl createrepo patch gcc lsof make python3-devel python3-unversioned-command " \
                      "rpm-build gdb-headless fuse-devel tcsh pciutils-devel kernel-devel".split()
            ret = subprocess.call(cmd, shell=False, stdout=subprocess.PIPE)

        # 安装麒麟V10 SP2系统依赖
        elif 'Kylin' in system_version:
            ret = subprocess.call(['yum', 'install', '-y', 'lsof'], shell=False, stdout=subprocess.PIPE)
            tcsh_rpm = os.path.join(mellanox_dirver_path, 'tcsh.rpm')
            if not os.path.exists(tcsh_rpm):
                print("tcsh dependency package doesn't exist, couldn't install mellanox driver.")
            else:
                subprocess.call(['yum', 'localinstall', '-y', tcsh_rpm], shell=False)

        # 安装openEuler系统Mellanox驱动依赖
        elif 'openEuler' in system_version:
            ret = subprocess.call(['yum', 'install', '-y', 'tcsh', 'pciutils-devel', 'fuse-devel', 'lsof',
                'createrepo', 'python3-devel', 'automake', 'rpm-build'], shell=False, stdout=subprocess.PIPE)
# 测试用：yum remove tcsh pciutils-devel fuse-devel lsof createrepo python3-devel automake rpm-build'


        # 安装CentOS V_7.6操作系统Mellanox驱动依赖包
        elif 'CentOS Linux release 7.6.1810' in system_version:
            ret = subprocess.call(['yum', 'install', '-y', 'tcl', 'tk', 'pciutils', 'lsof', 'tcsh'],
                shell=False, stdout=subprocess.PIPE)


        # 安装CentOS V_8.2操作系统Mellanox驱动依赖包
        elif 'CentOS Linux release 8.2.2004' in system_version:
            ret = subprocess.call(
                ['yum', 'install', '-y', 'gcc-gfortran', 'python36', 'kernel-modules-extra', 'tcl',
                 'tk', 'tcsh'], shell=False, stdout=subprocess.PIPE)


        # 安装麒麟信安系统Mellanox驱动依赖
        elif 'KylinSec' in system_version:
            ret = 0
            pass
        # 其他操作系统提示不支持，返回False
        else:
            print('Unsupported OS version')
            exit(255)
    # 如果yum命令执行失败，退出报错：
    if ret != 0:
        print('ERROR: Failed to install mellanox driver dependencies.')
        exit(255)


# 安装mellanox驱动
def install_mellanox_driver(mellanox_package_path):
    """
    :param mellanox_package_path: mellanox驱动包路径
    """
    # 切换当前目录到驱动路径
    os.chdir(mellanox_package_path)
    # 找到当前路径下的驱动包，如果没有找到，报错提示没找到驱动包；如果找到两个以上，报错提示找到多个
    driver_pkg = [f for f in os.listdir() if re.match(r'^MLNX_OFED_LINUX.*(tgz|tar.gz)$', f)]
    if not driver_pkg:
        print('No mellanox driver package found in {}'.format(mellanox_package_path))
        return False
    elif len(driver_pkg) > 1:
        print('Found multiple mellanox driver package in {}'.format(mellanox_package_path))
        return False
    driver_pkg = driver_pkg[0]
    # 把找到的驱动包复制到/tmp路径下
    ret = subprocess.call(['cp', driver_pkg, tempfile.gettempdir()], shell=False)
    if ret != 0:
        print('Failed to copy mellanox driver package to /tmp')
        return False
    print('Copy mellanox driver package to /tmp successfully.')
    # 切换到/tmp路径
    os.chdir(tempfile.gettempdir())
    # 解压驱动包
    subprocess.call(['tar', '-xzvf', driver_pkg], shell=False, stdout=subprocess.PIPE)
    # 切换到解压后的目录
    os.chdir(driver_pkg.replace('.tgz', '').replace('.tar.gz', ''))

    # 获取操作系统型号及内核型号，根据型号决定cmd具体内容
    with open('/etc/system-release', 'r') as f:
        system_version = f.read().strip()
        if 'Kylin Linux Advanced Server release V10 (Hydrogen)' in system_version:
            # 若系统内核版本为 5.10.0-153.5.v2305.ky10h.aarch64，则不需要编译可直接进行驱动安装
            kernel_version = subprocess.Popen(["uname", "-r"],
                shell=False, stdout=subprocess.PIPE).stdout.read().decode("utf-8").strip()
            if '5.10.0-153.5.v2305.ky10h.aarch64' in kernel_version:
                cmd = ["./mlnxofedinstall", "--distro", "kylin10hpc", "--force"]
            else:
                cmd = ["./mlnxofedinstall", "--add-kernel-support", "--distro", "kylin10hpc", "--force"]
        else:
            cmd = ["./mlnxofedinstall", "--add-kernel-support", "--force"]
    print("Installing Mellanox driver...")
    process = subprocess.Popen(cmd, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    output, error = process.communicate(timeout=1200)
    return_code = process.returncode
    output = output.decode("utf-8")
    error = error.decode("utf-8")
    # 安装驱动，捕获输出中是否有带"yum“字符串，若有，则执行该字符串，并重新安装驱动
    if return_code != 0:
        if 'yum' in output:
            # 根据需要选择是否打印驱动安装过程回显：
            print(output, error)
            # 正则匹配到output中的带“yum”的那一行
            yum_cmd = re.findall(r'.*yum install.*', output)[0]
            yum_cmd += " -y"
            print("Missing dependency, about to execute command to install: ", yum_cmd)
            ret = subprocess.call(yum_cmd.split(), shell=False)
            if ret != 0:
                print("Failed to install missing dependency.")
                return False
            # 重新执行本函数
            print("Reinstall the Mellanox driver...")
            install_mellanox_driver(mellanox_package_path)
        else:
            print(output, error)
            print("\nFailed to install mellanox driver.")
            return False
    print('Mellanox driver installed successfully.')
    # 根据需要选择是否打印驱动安装成功过程回显
    print(output, error)

    # 删除/tmp下的驱动包
    try:
        os.remove(os.path.join(tempfile.gettempdir(), driver_pkg))
        # 打印消息，提示/tmp下驱动包删除
        print('Mellanox driver package deleted from /tmp.')
    except FileNotFoundError:
        print('File not found.')
    except PermissionError:
        print('Permission denied.')
    except IsADirectoryError:
        print('That is a directory.')
    except Exception as e:
        print('An error occurred: {}'.format(str(e)))
    return True


# 驱动安装后基础配置
def mellanox_driver_post_config():
    # 检查是否已加载冲突的板载网卡模块，若存在，则移除
    process1 = subprocess.Popen(["lsmod"], shell=False, stdout=subprocess.PIPE)
    process2 = subprocess.Popen(["grep", "hns_roce_hw_v2"], shell=False, stdin=process1.stdout,
        stdout=subprocess.PIPE)
    process2.communicate(timeout=5)
    ret = process2.returncode
    if ret == 0:
        subprocess.call(['rmmod', 'hns_roce_hw_v2'], shell=False)

    process1 = subprocess.Popen(["lsmod"], shell=False, stdout=subprocess.PIPE)
    process2 = subprocess.Popen(["grep", "hns_roce"], shell=False, stdin=process1.stdout,
        stdout=subprocess.PIPE)
    process2.communicate(timeout=5)
    ret = process2.returncode
    if ret == 0:
        subprocess.call(['rmmod', 'hns_roce'], shell=False)

    # 设置下一次启动时更新驱动
    subprocess.call(['dracut', '-f'], shell=False)

    # 安装完成后执行以下命令加载驱动
    subprocess.call(['/etc/init.d/openibd', 'restart'], shell=False)

    # 配置网卡参数
    subprocess.call(['mst', 'start'], shell=False)
    # 获取网卡名称
    ret1 = subprocess.Popen("mlxconfig q".split(), shell=False, stdout=subprocess.PIPE)
    ret2 = subprocess.Popen(['grep', 'Device:'], shell=False, stdin=ret1.stdout, stdout=subprocess.PIPE)
    output = subprocess.Popen(['awk', '{print $2}'], shell=False, stdin=ret2.stdout, stdout=subprocess.PIPE).stdout.read().decode("utf-8")
    devices = output.split()
    for device in devices:
        # 设置网卡PF_LOG_BAR_SIZE参数值为8
        subprocess.call(['mlxconfig', '-d', device, '-y', 's', 'PF_LOG_BAR_SIZE=8'], shell=False)
        # 设置网卡PCI_WR_ORDERING参数值为1
        subprocess.call(['mlxconfig', '-d', device, '-y', 's', 'PCI_WR_ORDERING=1'], shell=False)


# 设置网卡模式为RoCE:
def set_network_mode_ib_to_roce():
    """
    若网卡支持IB和RoCE模式，切换为RoCE模式
    """
    # 获取网卡名称
    ret1 = subprocess.Popen("mlxconfig q".split(), shell=False, stdout=subprocess.PIPE)
    ret2 = subprocess.Popen(['grep', 'Device:'], shell=False, stdin=ret1.stdout, stdout=subprocess.PIPE)
    output = subprocess.Popen(['awk', '{print $2}'], shell=False, stdin=ret2.stdout, stdout=subprocess.PIPE).stdout.read().decode("utf-8")
    devices = output.split()
    for device in devices:
        # 查询当前网卡可支持的模式
        ret1 = subprocess.Popen(['mlxconfig', '-d', device, 'q'], shell=False, stdout=subprocess.PIPE)
        ret2 = subprocess.Popen(['grep', 'Description'], shell=False, stdin=ret1.stdout, stdout=subprocess.PIPE)
        ret = subprocess.call(['grep', 'EDR IB and 100GbE'], shell=False, stdin=ret2.stdout)
        if ret == 0:  # 若支持IB及RoCE，切换模式为RoCE
            subprocess.call(['mlxconfig', '-d', device, '-y', 's', 'LINK_TYPE_P1=2'], shell=False)


# 安装后校验
def post_install_check():
    # 检查驱动版本
    driver_version = subprocess.Popen(["ofed_info", "-n"], shell=False, stdout=subprocess.PIPE).stdout.read().decode("utf-8").strip()
    print('Mellanox driver version is {}'.format(driver_version))
    # 检查网卡状态
    netdev_status = subprocess.Popen(["ibdev2netdev"], shell=False, stdout=subprocess.PIPE).stdout.read().decode("utf-8").strip()
    print(netdev_status)


# 主函数
def main():
    validate_script_params()
    mellanox_package_path = ARGV_INPUT_DIC.get("mellanox_package_path")
    pre_install_check()
    install_mellanox_driver_depend(mellanox_package_path)
    install_mellanox_driver(mellanox_package_path)
    mellanox_driver_post_config()
    set_network_mode_ib_to_roce()
    post_install_check()
    print("Mellanox driver install successfully.\n \nPlease reboot machine to load new configurations.")
    exit(0)


main()
