#!/usr/bin/env python
# -*- coding: utf-8 -*-


"""
安装部署ldap客户端
该脚本需要传的参数如下：
ldap_login_pwd： openLDAP管理员密码
ldap_virtual_ip ：openLDAP虚拟IP地址，必填。
ldap_domain_name ：openLDAP服务访问域名名称，可选。例如：ldap01.huawei.com
"""

import sys
import os
import re
import subprocess
import socket
from time import sleep

REMOTE_PARAMETERS = " ".join(sys.argv[1:])  # 获取参数

# 参数示例
ARGV_EXAMPLE_DIC = {"ldap_login_pwd": "xxxx", "ldap_virtual_ip": "xxxx", "ldap_domain_name": "zzzz"}
ARGV_INPUT_DIC = ARGV_EXAMPLE_DIC

ldap_login_pwd = None
ldap_virtual_ip = None
ldap_domain_name = None
OS_TYPE = None


# 加固判断路径、文件是否存在
def path_hard_judgement():
    check_path_list = ["/etc/openldap", "/etc/pam.d/", "/etc/hosts"]
    for path in check_path_list:
        if not os.path.exists(path):
            print('The source file or folder does not exist.')
            exit(255)


# 参数展示样例
def argv_example_tips():
    message = "parameter example: "
    for key, value in ARGV_EXAMPLE_DIC.items():
        message = message + key + "=" + value + " "
    return message.strip()


# 校验传参个数
def validate_script_params():
    if len(REMOTE_PARAMETERS) == 0:
        print('Please enter parameters, ' + argv_example_tips())
        exit(255)
    tmp_argv = REMOTE_PARAMETERS.split(" ")
    for item in tmp_argv:
        argv = item.split("=")
        if len(argv) != 2:
            print('Invalid parameters format, ' + argv_example_tips())
            exit(255)
        if argv[0] not in ARGV_EXAMPLE_DIC.keys():
            print('Parameter [ ' + argv[0] + ' ] is not defined, ' + argv_example_tips())
            exit(255)
        if argv[1].strip() == "" and ARGV_EXAMPLE_DIC.get(argv[0]) == "xxxx":
            print('Parameter [ ' + argv[0] + ' ] cannot be empty.')
            exit(255)
        ARGV_INPUT_DIC[argv[0]] = argv[1]

    for key, value in ARGV_INPUT_DIC.items():
        if value == "xxxx":
            print('Parameter [ ' + key + ' ] cannot be empty.')
            exit(255)

    if len(ARGV_EXAMPLE_DIC) != len(ARGV_INPUT_DIC):
        print('Number of parameters does not match, ' + argv_example_tips())
        exit(255)


# 判断传参功能的准确性
def verify_input_argv():
    validate_script_params()
    tmp_argv = REMOTE_PARAMETERS.split(" ")
    # 密码复杂度校验
    global ldap_login_pwd
    ldap_login_pwd_argv = ARGV_INPUT_DIC.get("ldap_login_pwd")
    if len(ldap_login_pwd_argv) < 8 or not re.search(r"\d", ldap_login_pwd_argv) or not re.search(r"[a-zA-Z]",
                                                                                                  ldap_login_pwd_argv):
        print("Failed to verify the password.")
        exit(255)
    else:
        ldap_login_pwd = ldap_login_pwd_argv

    # 校验虚拟ip地址格式正确
    global ldap_virtual_ip
    ipv4_pattern = r'^((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$'
    vip_argv = ARGV_INPUT_DIC.get("ldap_virtual_ip")
    if re.match(ipv4_pattern, vip_argv):
        pass
    else:
        print(' ldap_virtual_ip {} Invalid parameters.'.format(vip_argv))
        exit(255)

    # 校验虚拟ip可达
    virtual_ip = (vip_argv, 22)
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.settimeout(2)
    result_vip = s.connect_ex(virtual_ip)
    if result_vip != 0:
        print(' ldap_slaver_ip {} is unreachable. '.format(vip_argv))
        exit(255)
    else:
        ldap_virtual_ip = vip_argv

    # 域名校验,先判断用户是否传入域名，若有则获取域名，若没有则默认域名
    global ldap_domain_name
    if len(tmp_argv) == 3:
        domain_argv = ARGV_INPUT_DIC.get("ldap_domain_name")
        if len(domain_argv) == 0 or len(domain_argv) > 254 or not re.match(
                "^((?!-)[A-Za-z0-9-]{1,63}(?<!-)\\.)+[A-Za-z]{2,6}$", domain_argv):
            print("Failed to verify the domain name. parameter example: ldapserver.huawei.com")
            exit(255)
        else:
            ldap_domain_name = domain_argv
    else:
        ldap_domain_name = 'ldapserver.huawei.com'


# 安装前操作(关闭防火墙、selinux、检查yum源状态、配置主备节点免密)
def check_host():
    verify_input_argv()
    # 查询os类型
    global OS_TYPE
    OS_TYPE = subprocess.check_output(["cat", "/etc/system-release"], shell=False).decode().strip()
    # 关闭selinux
    selinux_status = subprocess.check_output(["getenforce"], shell=False).decode().strip()
    if selinux_status == "Enforcing":
        subprocess.call("sed -i '/SELINUX/s/enforcing/disabled/' /etc/selinux/config".split(), shell=False)
        subprocess.call(["setenforce", "0"], shell=False)
    # 关闭防火墙
    subprocess.call(["systemctl", "stop", "firewalld"], shell=False)
    with open(os.devnull, 'w') as file:
        subprocess.call('firewall-cmd --reload'.split(), stdout=file, stderr=file)
        subprocess.call('systemctl disable firewalld'.split(), stdout=file, stderr=file)
    # 检查yum源是否可用
    yum_status = subprocess.call(["yum", "makecache"], shell=False)
    if yum_status != 0:
        print("yum is unavailable. please check the yum status.")
        exit(255)

    # 校验与ldap服务端的节点免密
    with open(os.devnull, 'w') as dev_null:
        res_passfree = subprocess.call('ssh root@{} -o PreferredAuthentications=publickey -o StrictHostKeyChecking=no '
                                       '"date"'.format(ldap_virtual_ip).split(), shell=False, stdout=dev_null,
                                       stderr=dev_null)
        if res_passfree != 0:
            print("Can't connect %s, Please configure password-free." % (ldap_virtual_ip))
            exit(255)


# 重启ldap服务
def restart_ladp():
    ret = subprocess.call('systemctl enable nscd.service'.split(), shell=False)
    if ret == 0:
        subprocess.call('systemctl restart nscd.service'.split(), shell=False)
    ret = subprocess.call('systemctl enable nslcd'.split(), shell=False)
    if ret == 0:
        subprocess.call('systemctl restart nslcd'.split(), shell=False)
    ret = subprocess.call('systemctl enable oddjobd'.split(), shell=False)
    if ret == 0:
        subprocess.call('systemctl restart oddjobd'.split(), shell=False)
    sleep(30)


########## pam文件修改 ##########
system_text = """#%PAM-1.0
# This file is auto-generated.
# User changes will be destroyed the next time authconfig is run.
auth        required      pam_env.so
auth        required      pam_faildelay.so delay=2000000
auth        sufficient    pam_unix.so nullok try_first_pass
auth        sufficient    pam_ldap.so use_first_pass
auth        requisite     pam_succeed_if.so uid >= 1200 quiet_success
account        required      pam_oddjob_mkhomedir.so umask=0077
auth        required      pam_deny.so

account     required      pam_unix.so broken_shadow
account     sufficient    pam_localuser.so
account     sufficient    pam_succeed_if.so uid < 1200 quiet
account     [default=bad success=ok user_unknown=ignore] pam_ldap.so
account     required      pam_permit.so
account     required      pam_oddjob_mkhomedir.so umask=0077

password    requisite     pam_pwquality.so try_first_pass local_users_only retry=3 authtok_type=
password    sufficient    pam_unix.so sha512 shadow nis nullok try_first_pass use_authtok
password    sufficient    pam_ldap.so use_authtok
password    required      pam_deny.so

session     optional      pam_keyinit.so revoke
session     required      pam_limits.so
-session     optional      pam_systemd.so
session     optional      pam_oddjob_mkhomedir.so umask=0077
session     [success=1 default=ignore] pam_succeed_if.so service in crond quiet use_uid
session     required      pam_unix.so
session     optional      pam_ldap.so"""

password_text = """#%PAM-1.0
# This file is auto-generated.
# User changes will be destroyed the next time authconfig is run.
auth        required      pam_env.so
auth        required      pam_faildelay.so delay=2000000
auth        sufficient    pam_unix.so nullok try_first_pass
auth        sufficient    pam_ldap.so use_first_pass
auth        requisite     pam_succeed_if.so uid >= 1200 quiet_success
account        required      pam_oddjob_mkhomedir.so umask=0077
auth        required      pam_deny.so

account     required      pam_unix.so broken_shadow
account     sufficient    pam_localuser.so
account     sufficient    pam_succeed_if.so uid < 1200 quiet
account     [default=bad success=ok user_unknown=ignore] pam_ldap.so
account     required      pam_permit.so
account     required      pam_oddjob_mkhomedir.so umask=0077

password    requisite     pam_pwquality.so try_first_pass local_users_only retry=3 authtok_type=
password    sufficient    pam_unix.so sha512 shadow nis nullok try_first_pass use_authtok
password    sufficient    pam_ldap.so use_authtok
password    required      pam_deny.so

session     optional      pam_keyinit.so revoke
session     required      pam_limits.so
-session     optional      pam_systemd.so
session     optional      pam_oddjob_mkhomedir.so umask=0077
session     [success=1 default=ignore] pam_succeed_if.so service in crond quiet use_uid
session     required      pam_unix.so
session     optional      pam_ldap.so"""


# 配置ldap客户端
def install_ldap_client1():
    check_host()
    # 使用yun安装ldap_client服务
    cmd = "yum -y install nss-pam-ldapd pam_ldap openldap-clients oddjob oddjob-mkhomedir nscd".split()
    install_ldap = subprocess.call(cmd, shell=False)
    if install_ldap != 0:
        print("Failed to install LDAP_client. Check the yum configuration.")
        exit(255)
    # 加固校验文件读取
    path_hard_judgement()
    if "Kylin" in OS_TYPE or "openEuler" in OS_TYPE or "KylinSec" in OS_TYPE:
        system_auth = '/etc/pam.d/system-auth'
        with open(system_auth, 'w') as f:
            f.write(system_text)

    if "Kylin" in OS_TYPE or "openEuler" in OS_TYPE or "KylinSec" in OS_TYPE:
        password_auth = '/etc/pam.d/password-auth'
        with open(password_auth, 'w') as f:
            f.write(password_text)

    # 配置hosts文件
    with open('/etc/hosts', 'r+') as file:
        str_file = file.read()
        if ldap_domain_name not in str_file and ldap_virtual_ip not in str_file:
            file.write("\n{} {}".format(ldap_virtual_ip, ldap_domain_name))

    # 创建ldap客户端证书存放目录
    if not os.path.exists('/etc/openldap/cacerts'):
        os.makedirs("/etc/openldap/cacerts")

    # copy服务端证书到客户端
    subprocess.call("scp -r root@{}:/etc/openldap/certs/ca.crt /etc/openldap/cacerts".format(ldap_virtual_ip).split(),
                    shell=False)
    # 配置权限证书权限
    os.chmod("/etc/openldap/cacerts", 0o755)
    os.chmod("/etc/openldap/cacerts/ca.crt", 0o644)


def install_ldap_client():
    install_ldap_client1()
    if "Kylin" in OS_TYPE or "openEuler" in OS_TYPE or "KylinSec" in OS_TYPE:
        nslcd_conf = '/etc/nslcd.conf'
        nslcd_text = """uid nslcd
gid ldap
uri ldaps://%s:636
base dc=huawei,dc=com
ssl yes
binddn cn=root,dc=huawei,dc=com
bindpw %s
tls_reqcert allow
tls_cacertdir /etc/openldap/cacerts
tls_cacert /etc/openldap/cacerts/ca.crt
nss_initgroups_ignoreusers ALLLOCAL
idle_timelimit 180
timelimit 120
bind_timelimit 120

""" % (ldap_domain_name, ldap_login_pwd)
        with open(nslcd_conf, 'w') as f:
            f.write(nslcd_text)

    if "Kylin" in OS_TYPE or "openEuler" in OS_TYPE or "KylinSec" in OS_TYPE:
        ldap_conf = '/etc/openldap/ldap.conf'
        ldap_text = """SASL_NOCANON    on
uri ldaps://%s
base dc=huawei,dc=com
ssl yes
TLS_REQCERT allow
TLS_CACERTDIR /etc/openldap/cacerts
TLS_CACERT /etc/openldap/cacerts/ca.crt""" % (ldap_virtual_ip)
        with open(ldap_conf, 'w') as f:
            f.write(ldap_text)
    os.chmod("/etc/nslcd.conf", 0o600)

    # 修改配置文件
    subprocess.call(['sed', '-i', 's/^passwd:.*$/passwd: files ldap/g', '/etc/nsswitch.conf'], shell=False)
    subprocess.call(['sed', '-i', 's/^shadow:.*$/shadow: files ldap/g', '/etc/nsswitch.conf'], shell=False)
    subprocess.call(['sed', '-i', 's/^group:.*$/group: files ldap/g', '/etc/nsswitch.conf'], shell=False)
    subprocess.call(
        ['sed', '-i', 's/positive-time-to-live\\tgroup.*$/positive-time-to-live\\tgroup\\t\\t600/g', '/etc/nscd.conf'],
        shell=False)
    subprocess.call(['sed', '-i', 's/#\\treload-count.*$/\\treload-count\\t\\t10/g', '/etc/nscd.conf'], shell=False)

    # 重启ldap服务
    restart_ladp()


def connect_ldap_server():
    # 连接ldap服务端
    ret1 = subprocess.Popen(['ldapsearch', '-b', 'dc=huawei,dc=com', '-D', 'cn=root,dc=huawei,dc=com', '-H',
                             'ldaps://%s:636' % ldap_domain_name, '-w', ldap_login_pwd], shell=False,
                            stdout=subprocess.PIPE)
    connect_ldap = subprocess.call(['grep', 'dn'], shell=False, stdin=ret1.stdout)
    if connect_ldap != 0:
        print("{}: Failed to connect to the LDAP server".format(socket.gethostname()))
    else:
        print("{}: Ldap Client Configuration Completed".format(socket.gethostname()))


if __name__ == "__main__":
    install_ldap_client()
    connect_ldap_server()
