#!/usr/bin/env python
# -*- coding: utf-8 -*-


"""
安装部署ldap服务端--master
该脚本需要传的参数如下：
ldap_login_pwd :  openLDAP管理员密码，必填；
ldap_slaver_ip :  openLDAP备节点IP地址，必填；
ldap_domain_name ：openLDAP服务访问域名名称，可选。例如：ldap01.huawei.com
ldap_virtual_ip ：openLDAP虚拟IP地址，必填。
"""

import glob
import sys
import os
import re
import shutil
import subprocess
import socket

REMOTE_PARAMETERS = " ".join(sys.argv[1:])  # 获取参数

# 参数示例
ARGV_EXAMPLE_DIC = {"ldap_login_pwd": "xxxx", "ldap_slaver_ip": "xxxx", "ldap_virtual_ip": "xxxx", "ldap_domain_name": "zzzz"}
ARGV_INPUT_DIC = ARGV_EXAMPLE_DIC

ldap_login_pwd = None
network_port = None
OS_TYPE = None
ldap_master_ip = None
ldap_slaver_ip = None
ldap_virtual_ip = None
ldap_domain_name = None


# 加固判断路径、文件是否存在
def path_hard_judgement():
    check_path_list = ["/etc/keepalived", "/etc/openldap", "/etc/rsyslog.conf",
                       "/etc/sysconfig", "/etc/logrotate.d", "/etc/openldap/slapd.d",
                       "/usr/lib/systemd/system/slapd.service"]
    for path in check_path_list:
        if not os.path.exists(path):
            print('{} : The source file or folder does not exist.'.format(path))        
            exit(255)


# 参数展示样例
def argv_example_tips():
    message = "parameter example: "
    for key, value in ARGV_EXAMPLE_DIC.items():
        message = message + key + "=" + value + " "
    return message.strip()


# 校验传参个数
def validate_script_params():
    if len(REMOTE_PARAMETERS) == 0:
        print('Please enter parameters, ' + argv_example_tips())
        exit(255)
    tmp_argv = REMOTE_PARAMETERS.split(" ")
    for item in tmp_argv:
        argv = item.split("=")
        if len(argv) != 2:
            print('Invalid parameters format, ' + argv_example_tips())
            exit(255)
        if argv[0] not in ARGV_EXAMPLE_DIC.keys():
            print('Parameter [ ' + argv[0] + ' ] is not defined, ' + argv_example_tips())
            exit(255)
        if argv[1].strip() == "" and ARGV_EXAMPLE_DIC.get(argv[0]) == "xxxx":
            print('Parameter [ ' + argv[0] + ' ] cannot be empty.')
            exit(255)
        ARGV_INPUT_DIC[argv[0]] = argv[1]

    for key, value in ARGV_INPUT_DIC.items():
        if value == "xxxx":
            print('Parameter [ ' + key + ' ] cannot be empty.')
            exit(255)

    if len(ARGV_EXAMPLE_DIC) != len(ARGV_INPUT_DIC):
        print('Number of parameters does not match, ' + argv_example_tips())
        exit(255)


# 判断传参功能的准确性
def verify_input_argv():
    validate_script_params()
    tmp_argv = REMOTE_PARAMETERS.split(" ")
    # 密码复杂度校验
    global ldap_login_pwd
    ldap_login_pwd_argv = ARGV_INPUT_DIC.get("ldap_login_pwd")
    if len(ldap_login_pwd_argv) < 8 or not re.search(r"\d", ldap_login_pwd_argv) or not re.search(r"[a-zA-Z]",
                                                                                                  ldap_login_pwd_argv):
        print("Failed to verify the password.")
        exit(255)
    else:
        ldap_login_pwd = ldap_login_pwd_argv

    # 校验备节点ip、虚拟ip地址格式正确
    global ldap_slaver_ip, ldap_virtual_ip
    ipv4_pattern = r'^((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$'
    check_ip = [ARGV_INPUT_DIC.get("ldap_slaver_ip"), ARGV_INPUT_DIC.get("ldap_virtual_ip")]
    for item in check_ip:
        if not re.match(ipv4_pattern, item):
            print(' {} Incorrect IP address format.'.format(item))
            exit(255)

    # 校验备节点ip地址可达
    slaver_ip_argv = ARGV_INPUT_DIC.get("ldap_slaver_ip")
    slaver_ip = (slaver_ip_argv, 22)
    s1 = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s1.settimeout(2)
    result_slaver_ip = s1.connect_ex(slaver_ip)
    if result_slaver_ip != 0:
        print(' ldap_slaver_ip {} is unreachable.'.format(slaver_ip_argv))
        exit(255)
    else:
        ldap_slaver_ip = slaver_ip_argv

    # 获取虚拟ip
    ldap_virtual_ip = ARGV_INPUT_DIC.get("ldap_virtual_ip")

    # 域名校验,先判断用户是否传入域名，若有则获取域名，若没有则默认域名
    global ldap_domain_name
    if len(tmp_argv) == 4:
        domain_argv = ARGV_INPUT_DIC.get("ldap_domain_name")
        if len(domain_argv) == 0 or len(domain_argv) > 254 or not re.match(
                "^((?!-)[A-Za-z0-9-]{1,63}(?<!-)\\.)+[A-Za-z]{2,6}$", domain_argv):
            print("Failed to verify the domain name. parameter example: ldapserver.huawei.com")
            exit(255)
        else:
            ldap_domain_name = domain_argv
    else:
        ldap_domain_name = 'ldapserver.huawei.com'


# 安装前操作(关闭防火墙、selinux、检查yum源状态、配置主备节点免密)
def check_host():
    # 查询os类型
    global OS_TYPE
    OS_TYPE = subprocess.check_output(["cat", "/etc/system-release"], shell=False).decode().strip()
    # 查询本机ip即ldap主节点ip
    global ldap_master_ip
    ldap_master_ip = socket.gethostbyname(socket.gethostname())
    # 获取本机网口名
    global network_port
    process1 = subprocess.Popen(["ip", "a"], shell=False, stdout=subprocess.PIPE)
    process2 = subprocess.Popen(["grep", "-B", "2", ldap_master_ip], shell=False, stdin=process1.stdout,
        stdout=subprocess.PIPE)
    process3 = subprocess.Popen(["head", "-n", "1"], shell=False, stdin=process2.stdout, stdout=subprocess.PIPE)
    network_port = subprocess.check_output(["awk", "-F:", "{print $2}"], shell=False,
        stdin=process3.stdout).decode().strip()
    # 关闭selinux
    selinux_status = subprocess.check_output(["getenforce"], shell=False).decode().strip()
    if selinux_status == "Enforcing":
        subprocess.call(["sed", "-i", "/SELINUX/s/enforcing/disabled/", "/etc/selinux/config"], shell=False)
        subprocess.call(["setenforce", "0"], shell=False)
    # 关闭防火墙
    with open(os.devnull, 'w') as dev_null:
        subprocess.call(["firewall-cmd", "--reload"], shell=False, stdout=dev_null, stderr=dev_null)
        subprocess.call(["systemctl", "disable", "firewalld"], shell=False, stdout=dev_null, stderr=dev_null)
    # 检查yum源是否可用
    yum_status = subprocess.call(["yum", "makecache"], shell=False)
    if yum_status != 0:
        print("yum is unavailable. please check the yum status.")
        exit(255)

    # 校验主备节点免密
    with open(os.devnull, 'w') as dev_null:
        res_passfree = subprocess.call('ssh root@{} -o PreferredAuthentications=publickey -o StrictHostKeyChecking=no '
            '"date"'.format(ldap_slaver_ip).split(), shell=False, stdout=dev_null, stderr=dev_null)
        if res_passfree != 0:
            print("Can't connect %s, Please configure password-free." % (ldap_slaver_ip))
            exit(255)


# 安装ldap服务端
def install_ldap_server():
    check_host()
    # 安装前检查环境上是否有ldap服务，有则删除，避免安装失败
    rpm_status = subprocess.check_output(["rpm", "-qa", "openldap"], shell=False).decode().strip()
    if len(rpm_status) != 0:
        subprocess.call(["yum", "remove", "-y", "openldap-servers"], shell=False)
        if os.path.exists("/var/lib/ldap"):
            shutil.rmtree("/var/lib/ldap")
        if os.path.exists("/etc/openldap"):
            shutil.rmtree("/etc/openldap")
        subprocess.call(["userdel", "ldap"], shell=False)
        files = glob.glob("/var/log/slapd.log*")
        for file in files:
            os.remove(file)
    # 开始安装ldap服务
    os_type_group1 = ["Kylin", "openEuler", "KylinSec", "Red Hat Enterprise Linux"]
    if any(item in OS_TYPE for item in os_type_group1):
        cmd = "yum -y install openldap openldap-clients openldap-servers openldap-devel expect net-tools " \
            "keepalived".split()
        subprocess.call(cmd, shell=False)
    elif "CentOS" in OS_TYPE:
        cmd = "yum -y install openldap compat-openldap openldap-clients openldap-servers " \
            "openldap-servers-sql openldap-devel expect net-tools keepalived".split()
        subprocess.call(cmd, shell=False)
    else:
        print("Failed to install the LDAP service.")
        exit(255)
    # 加固校验文件读取
    path_hard_judgement()


def check_keepalived_conf():
    """
    return: 如果有portal返回1，如果同时有portal和ldap返回2，没有返回0
    """
    with open('/etc/keepalived/keepalived.conf', 'r') as file:
        content = file.read()
        if 'portal' in content and 'ldap' in content:
            return 2
        elif 'portal' in content:
            return 1
        else:
            return 0

########## slapd.ldif文件模板 ##########
verify_input_argv()
install_ldap_server()
sladp_passwd = subprocess.check_output(["slappasswd", "-s", "{}".format(ldap_login_pwd)]).decode().strip()
redhat_text_slapd = '''dn: cn=config
objectClass: olcGlobal
cn: config
olcArgsFile: /var/run/openldap/slapd.args
olcPidFile: /var/run/openldap/slapd.pid

dn: cn=schema,cn=config
objectClass: olcSchemaConfig
cn: schema

include: file:///etc/openldap/schema/core.ldif
include: file:///etc/openldap/schema/cosine.ldif
include: file:///etc/openldap/schema/nis.ldif
include: file:///etc/openldap/schema/inetorgperson.ldif

dn: olcDatabase=frontend,cn=config
objectClass: olcDatabaseConfig
objectClass: olcFrontendConfig
olcDatabase: frontend

dn: olcDatabase=config,cn=config
objectClass: olcDatabaseConfig
olcDatabase: config
olcAccess: to * by dn.base="gidNumber=0+uidNumber=0,cn=peercred,cn=external,c
 n=auth" manage by * none

dn: olcDatabase=monitor,cn=config
objectClass: olcDatabaseConfig
olcDatabase: monitor
olcAccess: to * by dn.base="gidNumber=0+uidNumber=0,cn=peercred,cn=external,c
 n=auth" read by dn.base="cn=root,dc=huawei,dc=com" read by * none

dn: olcDatabase=hdb,cn=config
objectClass: olcDatabaseConfig
objectClass: olcHdbConfig
olcDatabase: hdb
olcSuffix: dc=huawei,dc=com
olcRootDN: cn=root,dc=huawei,dc=com
olcRootPW: {}
olcDbDirectory: /var/lib/ldap
olcDbIndex: objectClass eq,pres
olcDbIndex: ou,cn,mail,surname,givenname eq,pres,sub'''.format(sladp_passwd)


openEuler_text_slapd = '''dn: cn=config
objectClass: olcGlobal
cn: config
olcArgsFile: /var/run/openldap/slapd.args
olcPidFile: /var/run/openldap/slapd.pid

dn: cn=schema,cn=config
objectClass: olcSchemaConfig
cn: schema

include: file:///etc/openldap/schema/core.ldif
include: file:///etc/openldap/schema/cosine.ldif
include: file:///etc/openldap/schema/nis.ldif
include: file:///etc/openldap/schema/inetorgperson.ldif

dn: olcDatabase=frontend,cn=config
objectClass: olcDatabaseConfig
objectClass: olcFrontendConfig
olcDatabase: frontend

dn: olcDatabase=config,cn=config
objectClass: olcDatabaseConfig
olcDatabase: config
olcAccess: to * by dn.base="gidNumber=0+uidNumber=0,cn=peercred,cn=external,c
 n=auth" manage by * none

dn: olcDatabase=monitor,cn=config
objectClass: olcDatabaseConfig
olcDatabase: monitor
olcAccess: to * by dn.base="gidNumber=0+uidNumber=0,cn=peercred,cn=external,c
 n=auth" read by dn.base="cn=root,dc=huawei,dc=com" read by * none

dn: olcDatabase=mdb,cn=config
objectClass: olcDatabaseConfig
objectClass: olcMdbConfig
olcDatabase: mdb
olcSuffix: dc=huawei,dc=com
olcRootDN: cn=root,dc=huawei,dc=com
olcRootPW: {}
olcDbDirectory: /var/lib/ldap
olcDbIndex: objectClass eq,pres
olcDbIndex: ou,cn,mail,surname,givenname eq,pres,sub'''.format(sladp_passwd)

########## 创建用户组文件模板 ##########
text_base = '''dn: dc=huawei,dc=com
o: huawei com
dc: huawei
objectClass: top
objectClass: dcObject
objectclass: organization

dn: cn=root,dc=huawei,dc=com
cn: root
objectClass: organizationalRole
description: Directory Manager

dn: ou=People,dc=huawei,dc=com
ou: People
objectClass: top
objectClass: organizationalUnit

dn: ou=Group,dc=huawei,dc=com
ou: Group
objectClass: top
objectClass: organizationalUnit'''

########## 关闭匿名认证访问文件模板 ##########
disable_anon_text = """dn: cn=config
changetype: modify
add: olcDisallows
olcDisallows: bind_anon
"""

########## 非安全端口主备同步sync-ha.ldif文件模板 ##########
openEuler_text_sync_ha = '''dn: olcDatabase={2}mdb,cn=config
changetype: modify
add: olcSyncRepl
olcSyncRepl: rid=001
             provider=ldap://%s
             bindmethod=simple
             binddn="cn=root,dc=huawei,dc=com"
             credentials=%s
             searchbase="dc=huawei,dc=com"
             scope=sub
             schemachecking=on
             type=refreshAndPersist
             retry="30 5 300 3"
             interval=00:00:05:0
olcSyncrepl: rid=002
             provider=ldap://%s
             bindmethod=simple
             binddn="cn=root,dc=huawei,dc=com"
             credentials=%s
             searchbase="dc=huawei,dc=com"
             scope=sub
             schemachecking=on
             type=refreshAndPersist
             retry="30 5 300 3"
             interval=00:00:05:00
-
add: olcMirrorMode
olcMirrorMode: TRUE''' % (ldap_master_ip, ldap_login_pwd, ldap_slaver_ip, ldap_login_pwd)

redhat_text_sync_ha = '''dn: olcDatabase={2}hdb,cn=config
changetype: modify
add: olcSyncRepl
olcSyncRepl: rid=001
             provider=ldap://%s
             bindmethod=simple
             binddn="cn=root,dc=huawei,dc=com"
             credentials=%s
             searchbase="dc=huawei,dc=com"
             scope=sub
             schemachecking=on
             type=refreshAndPersist
             retry="30 5 300 3"
             interval=00:00:05:0
olcSyncrepl: rid=002
             provider=ldap://%s
             bindmethod=simple
             binddn="cn=root,dc=huawei,dc=com"
             credentials=%s
             searchbase="dc=huawei,dc=com"
             scope=sub
             schemachecking=on
             type=refreshAndPersist
             retry="30 5 300 3"
             interval=00:00:05:00
-
add: olcMirrorMode
olcMirrorMode: TRUE''' % (ldap_master_ip, ldap_login_pwd, ldap_slaver_ip, ldap_login_pwd)

########## 安全端口主备同步sync-ha.ldif文件模板 ##########
openEuler_text_sync_ha_sec = """dn: olcDatabase={2}mdb,cn=config
changetype: modify
replace: olcSyncRepl
olcSyncRepl: rid=001
             provider=ldaps://%s:636
             bindmethod=simple
             binddn="cn=root,dc=huawei,dc=com"
             credentials=%s
             searchbase="dc=huawei,dc=com"
             tls_reqcert=allow
             scope=sub
             schemachecking=on
             type=refreshAndPersist
             retry="30 5 300 3"
             interval=00:00:05:0
olcSyncrepl: rid=002
             provider=ldaps://%s:636
             bindmethod=simple
             binddn="cn=root,dc=huawei,dc=com"
             credentials=%s
             searchbase="dc=huawei,dc=com"
             tls_reqcert=allow
             scope=sub
             schemachecking=on
             type=refreshAndPersist
             retry="30 5 300 3"
             interval=00:00:05:00
-
""" % (ldap_master_ip, ldap_login_pwd, ldap_slaver_ip, ldap_login_pwd)
redhat_text_sync_ha_sec = """dn: olcDatabase={2}hdb,cn=config
changetype: modify
replace: olcSyncRepl
olcSyncRepl: rid=001
             provider=ldaps://%s:636
             bindmethod=simple
             binddn="cn=root,dc=huawei,dc=com"
             credentials=%s
             searchbase="dc=huawei,dc=com"
             tls_reqcert=allow
             scope=sub
             schemachecking=on
             type=refreshAndPersist
             retry="30 5 300 3"
             interval=00:00:05:0
olcSyncrepl: rid=002
             provider=ldaps://%s:636
             bindmethod=simple
             binddn="cn=root,dc=huawei,dc=com"
             credentials=%s
             searchbase="dc=huawei,dc=com"
             tls_reqcert=allow
             scope=sub
             schemachecking=on
             type=refreshAndPersist
             retry="30 5 300 3"
             interval=00:00:05:00
-
""" % (ldap_master_ip, ldap_login_pwd, ldap_slaver_ip, ldap_login_pwd)

########## 主备同步mod_syncprov.ldif文件模板 ##########
text_mod_syncprov = '''dn: cn=module,cn=config
objectClass: olcModuleList
cn: module
olcModulePath: /usr/lib64/openldap
olcModuleLoad: syncprov.la'''

########## 主备同步serverid.ldif文件模板 ##########
redhat_text_serverid = '''dn: cn=config
changetype: modify
add: olcServerId
olcServerId: 1 ldap://{}'''.format(ldap_master_ip)
openEuler_text_serverid = '''dn: cn=config
changetype: modify
add: olcServerId
olcServerId: 1'''

########## 主备同步syncprov.ldif文件模板 ##########
openEuler_text_syncprov = '''dn: olcOverlay=syncprov,olcDatabase={2}mdb,cn=config
objectClass: olcOverlayConfig
objectClass: olcSyncProvConfig
olcOverlay: syncprov
olcSpSessionLog: 100'''
redhat_text_syncprov = '''dn: olcOverlay=syncprov,olcDatabase={2}hdb,cn=config
objectClass: olcOverlayConfig
objectClass: olcSyncProvConfig
olcOverlay: syncprov
olcSpSessionLog: 100'''

########## keepalived文件模板 ##########
# keepalived.conf文件中什么都没配：
text_keepalived = ''' ! Configuration File for keepalived
global_defs {
   router_id %s
}
vrrp_script check_ldap_server_status {
   script "/etc/openldap/slapd_check.sh"
   interval 3
   weight -5
}
vrrp_instance VI_LDAP {
   state MASTER
   interface %s
   virtual_router_id 204
   priority 100
   advert_int 1
   virtual_ipaddress {
     %s
   }
   notify_master "/etc/openldap/slapd_master.sh"
   notify_backup "/etc/openldap/slapd_master.sh"
   notify_stop "/etc/openldap/slapd_stop.sh"
   track_script {
      check_ldap_server_status
   }
}''' % (ldap_master_ip, network_port, ldap_virtual_ip)

# keepalived.conf文件中配了portal：
exist_portal_text = '''
vrrp_script check_ldap_server_status {
   script "/etc/openldap/slapd_check.sh"
   interval 3
   weight -5
}
vrrp_instance VI_LDAP {
   state MASTER
   interface %s
   virtual_router_id 204
   priority 100
   advert_int 1
   virtual_ipaddress {
     %s
   }
   notify_master "/etc/openldap/slapd_master.sh"
   notify_backup "/etc/openldap/slapd_master.sh"
   notify_stop "/etc/openldap/slapd_stop.sh"
   track_script {
      check_ldap_server_status
   }
}''' % (network_port, ldap_virtual_ip)

########## 日志配置文件模板 ##########
text_loglevel = '''dn: cn=config
changetype: modify
replace: olcLogLevel
olcLogLevel: stats'''

text_log = '''/var/log/slapd.log{
daily
rotate 5
copytruncate
nocompress
dateext
missingok
}'''


# 创建ldap相关文件(slapd.ldif, base.ldif, loglevel.ldif, slapd)
def create_slapd_ldif():
    if "Red Hat Enterprise Linux" in OS_TYPE or "KylinSec" in OS_TYPE:
        text_slapd = redhat_text_slapd
    else:
        text_slapd = openEuler_text_slapd
    with open("/etc/openldap/slapd.ldif", 'w') as f:
        f.write(text_slapd)

    # 创建用户及用户组文件
    with open("/etc/openldap/base.ldif", 'w') as f:
        f.write(text_base)

    # 创建日志配置文件
    with open("/etc/openldap/loglevel.ldif", 'w') as f:
        f.write(text_loglevel)

    # 配置ldap日志自动切割转储文件
    with open("/etc/logrotate.d/slapd", 'w') as f:
        f.write(text_log)

    # 配置ldap主备同步文件
    with open("/etc/openldap/mod_syncprov.ldif", 'w') as f:
        f.write(text_mod_syncprov)

    if "Red Hat Enterprise Linux" in OS_TYPE:
        text_serverid = redhat_text_serverid
    else:
        text_serverid = openEuler_text_serverid
    with open("/etc/openldap/serverid.ldif", 'w') as f:
        f.write(text_serverid)

    if "Kylin" in OS_TYPE or "openEuler" in OS_TYPE:
        text_syncprov = openEuler_text_syncprov
    else:
        text_syncprov = redhat_text_syncprov
    with open("/etc/openldap/syncprov.ldif", 'w') as f:
        f.write(text_syncprov)

    if "Kylin" in OS_TYPE or "openEuler" in OS_TYPE or "KylinSec" in OS_TYPE:
        text_sync_ha = openEuler_text_sync_ha
    elif "CentOS" in OS_TYPE or "Red Hat Enterprise Linux" in OS_TYPE:
        with open('/etc/sysconfig/slapd', 'a') as f:
            f.write('SLAPD_LDAPI=yes')
        text_sync_ha = redhat_text_sync_ha
    with open("/etc/openldap/sync-ha.ldif", 'w') as f:
        f.write(text_sync_ha)


# 配置ldap服务
def conf_ldap_server():
    # 备份slapd.d文件
    os.rename("/etc/openldap/slapd.d", "/etc/openldap/slapd.d.bak")
    os.mkdir("/etc/openldap/slapd.d")
    create_slapd_ldif()
    # 修改ldap相关文件
    result = subprocess.call(
        ["slapadd", "-n", "0", "-F", "/etc/openldap/slapd.d", "-l", "/etc/openldap/slapd.ldif"], shell=False)
    if result != 0:
        exit(255)
    # 配置ldap数据库
    if "Red Hat Enterprise Linux" in OS_TYPE or "CentOS" in OS_TYPE:
        shutil.copy("/usr/share/openldap-servers/DB_CONFIG.example", "/var/lib/ldap/DB_CONFIG")
    subprocess.call("chown ldap:ldap -R /etc/openldap/slapd.d".split(), shell=False)
    subprocess.call("chown ldap:ldap -R /var/lib/ldap".split(), shell=False)
    subprocess.call("chmod 700 -R /var/lib/ldap".split(), shell=False)
    # 加载ldap文件
    source_ldap = subprocess.call(["slaptest", "-u"], shell=False)
    if source_ldap != 0:
        print("Basic configuration of OpenLDAP is failed.")
        exit(255)
    # 启动ldap服务
    returncode = subprocess.call(["systemctl", "enable", "slapd"], shell=False)
    if returncode == 0:
        subprocess.call(["systemctl", "start", "slapd"], shell=False)
    # 查看ldap默认侦听端口是否为389
    ldap_port = subprocess.check_output(["netstat", "-antup"], shell=False).decode().strip()
    if "389" not in ldap_port:
        print("ERROR: The slapd service listening port is abnormal.")
        exit(255)
    # 修改slapd.service文件，增加ldap服务可靠性
    slapd_service = '/usr/lib/systemd/system/slapd.service'
    with open(slapd_service, 'r') as f:
        text_slapd_service = f.read()
        if 'LimitNOFILE' not in text_slapd_service:
            subprocess.call(
                ["sed", "-i", "/Service/a LimitNOFILE=65535\\nRestart=always\\nRestartSec=30s\\nLimitNPROC=65535",
                slapd_service], shell=False)
    returncode = subprocess.call(["systemctl", "daemon-reload"], shell=False)
    if returncode == 0:
        subprocess.call(["systemctl", "restart", "slapd"], shell=False)
    ldap_status = subprocess.check_output(["systemctl", "status", "slapd"], shell=False,
        universal_newlines=True)
    if "Active: active (running)" not in ldap_status:
        print("Slapd service is running abnormally.")
        exit(255)


# 在主节点导入ldif文件，配置同步
def import_ldif_files():
    result_mod = subprocess.call(
        ["ldapadd", "-Y", "EXTERNAL", "-H", "ldapi:///", "-f", "/etc/openldap/mod_syncprov.ldif"], shell=False)
    if result_mod != 0:
        print('Failed to import mod_syncprov.ldif to the database.')
        exit(255)
    result_serverid = subprocess.call(
        ["ldapadd", "-Y", "EXTERNAL", "-H", "ldapi:///", "-f", "/etc/openldap/serverid.ldif"], shell=False)
    if result_serverid != 0:
        print('Failed to import serverid.ldif to the database.')
        exit(255)
    result_syncprov = subprocess.call(
        ["ldapadd", "-Y", "EXTERNAL", "-H", "ldapi:///", "-f", "/etc/openldap/syncprov.ldif"], shell=False)
    if result_syncprov != 0:
        print('Failed to import syncprov.ldif to the database.')
        exit(255)
    result_sync_ha = subprocess.call(
        ["ldapadd", "-Y", "EXTERNAL", "-H", "ldapi:///", "-f", "/etc/openldap/sync-ha.ldif"], shell=False)
    if result_sync_ha != 0:
        print('Failed to import sync-ha.ldif to the database.')
        exit(255)


# 导入用户及用户组到ldap数据库、配置ldap日志功能
def conf_ldap_database():
    conf_ldap_server()
    # 导入基础数据库
    import_base = subprocess.call(["ldapadd", "-x", "-w", ldap_login_pwd, "-D", "cn=root,dc=huawei,dc=com", "-f",
        "/etc/openldap/base.ldif"], shell=False)
    if import_base != 0:
        print('Failed to import base.ldif to the database.')
        exit(255)
    process1 = subprocess.Popen(["ldapsearch", "-x", "-b", "dc=huawei,dc=com", "-H", "ldap://127.0.0.1"],
        shell=False, stdout=subprocess.PIPE)
    process1.communicate(timeout=10)
    check_base = process1.returncode
    if check_base != 0:
        print('Failed to search LDAP information.')
        exit(255)
    # 配置ldap日志功能
    with open('/etc/rsyslog.conf', 'r+') as f:
        text_rsylog = f.read()
        if '/var/log/slapd.log' not in text_rsylog:
            f.write('local4.* /var/log/slapd.log \n')
    import_log = subprocess.call(
        ["ldapmodify", "-Y", "EXTERNAL", "-H", "ldapi:///", "-f", "/etc/openldap/loglevel.ldif"], shell=False)
    if import_log != 0:
        print('Failed to import loglevel.ldif to the database.')
        exit(255)
    # 配置OpenLDAP日志自动切割转储策略
    subprocess.call(["logrotate", "-f", "/etc/logrotate.d/slapd"], shell=False)
    # 配置完成后重启ldap服务
    returncode = subprocess.call(["systemctl", "restart", "rsyslog.service"], shell=False)
    if returncode == 0:
        subprocess.call(["systemctl", "restart", "slapd.service"], shell=False)
    # 在主节点导入ldif文件，配置同步
    import_ldif_files()
    # 重启ldap服务
    subprocess.call(["systemctl", "restart", "slapd.service"], shell=False)
    ldap_status = subprocess.check_output(["systemctl", "status", "slapd"], shell=False,
        universal_newlines=True)
    if "Active: active (running)" not in ldap_status:
        print("Slapd service is running abnormally.")
        exit(255)
    # 主节点查看已存在用户
    process1 = subprocess.Popen(["ldapsearch", "-x", "-D", "cn=root,dc=huawei,dc=com", "-w", ldap_login_pwd, "-b",
        "dc=huawei,dc=com", "-H", "ldap://%s:389" % ldap_master_ip], shell=False, stdout=subprocess.PIPE)
    process1.communicate(timeout=10)
    check_user = process1.returncode
    if check_user != 0:
        print('Failed to search ldap user on the master node.')
        exit(255)


# 创建keepalived相关文件
def create_keepalived_file():
    slapd_master = "/etc/openldap/slapd_master.sh"
    text_slapd_master = '''#!/bin/bash
systemctl start slapd.service'''
    with open(slapd_master, 'w') as f:
        f.write(text_slapd_master)

    slapd_stop = "/etc/openldap/slapd_stop.sh"
    text_slapd_stop = '''#!/bin/bash
systemctl stop slapd.service'''
    with open(slapd_stop, 'w') as f:
        f.write(text_slapd_stop)

    slapd_check = "/etc/openldap/slapd_check.sh"
    text_slapd_check = """#!/bin/bash
ldapPid=$(ps -ef |grep slapd|grep -v grep|awk '{print $2}'|grep -v PID)
if [ "$ldapPid" == "" ]; then
   systemctl start slapd.service
   exit 1
else
   exit 0
fi"""
    with open(slapd_check, 'w') as f:
        f.write(text_slapd_check)

    os.chmod(slapd_master, 0o645)
    os.chmod(slapd_stop, 0o645)
    os.chmod(slapd_check, 0o645)

    keepalived_conf = "/etc/keepalived/keepalived.conf"
    if not os.path.exists(keepalived_conf) or not os.path.isfile(keepalived_conf):
        print("{}: No such file".format(keepalived_conf))
        exit(255)
    return_code = check_keepalived_conf()
    if return_code == 0:   # keepalived.conf文件为原始文件
        with open(keepalived_conf, 'w') as f:
            f.write(text_keepalived)
    elif return_code == 1:   # keepalived.conf文件有portal
        with open(keepalived_conf, "a") as f:
            f.write(exist_portal_text)
        subprocess.call(["sed", "-i", "/^$/{N;/\\n$/D}", keepalived_conf])
    elif return_code == 2:   # keepalived.conf文件有portal和ldap
        # 删除ldap相关配置
        subprocess.call(["sed", "-i", "-r", "/vrrp_script check_ldap_server_status/,/}/ d", keepalived_conf])
        subprocess.call(["sed", "-i", "-r", "/track_script/,/}/ d", keepalived_conf])
        subprocess.call(["sed", "-i", "-r", "/virtual_ipaddress/,/}/ d", keepalived_conf])
        subprocess.call(["sed", "-i", "-r", "/vrrp_instance VI_LDAP/,/}/ d", keepalived_conf])
        with open(keepalived_conf, "a") as f:
            f.write(exist_portal_text)
        subprocess.call(["sed", "-i", "/^$/{N;/\\n$/D}", keepalived_conf])

    os.chmod("/etc/keepalived/keepalived.conf", 0o644)
    subprocess.call(["systemctl", "enable", "keepalived.service"], shell=False)
    subprocess.call(["systemctl", "restart", "keepalived.service"], shell=False)


# 生成SSL证书
def generate_ssl_certificate():
    os.system('expect -c "set timeout -1;\n spawn openssl genrsa -aes256 -out /etc/openldap/certs/ca.key 3072\n '
              '\expect {\n*ca.key* {send -- %s\\r;exp_continue;}\n*Verifying* {send -- %s\\r;exp_continue;}\n eof {exit 0;}\n}"' % (
                  ldap_login_pwd, ldap_login_pwd))
    os.system(
        'expect -c "set timeout -1;\n spawn openssl req -x509 -new -nodes -key /etc/openldap/certs/ca.key -sha256 -subj "/C=CN/ST=GuangDong/L=ShenZhen/O=HW/OU=IT/CN=%s" -days 7300 -out /etc/openldap/certs/ca.crt\n '
        'expect {\n *ca.key* {send -- %s\\r;exp_continue;}\n eof {exit 0;}\n}"' % (ldap_domain_name, ldap_login_pwd))
    subprocess.call(["openssl", "genrsa", "-out", "/etc/openldap/certs/ldap.key", "3072"], shell=False)
    subprocess.call(["openssl", "req", "-new", "-subj", "/C=CN/ST=GuangDong/L=ShenZhen/O=HW/OU=IT/CN=%s" %
        ldap_domain_name, "-key", "/etc/openldap/certs/ldap.key", "-out", "/etc/openldap/certs/ldap.csr"], shell=False)
    os.system(
        'expect -c "set timeout -1;\n spawn openssl x509 -req -in /etc/openldap/certs/ldap.csr -out /etc/openldap/certs/ldap.crt -days 3650 -CAcreateserial -CA /etc/openldap/certs/ca.crt -CAkey /etc/openldap/certs/ca.key -CAserial /etc/openldap/certs/serial -extfile /etc/openldap/certs/my-ssl.conf\n '
        'expect {\n *ca.key* {send -- %s\\r;exp_continue;} \neof {exit 0;}\n}"' % (ldap_login_pwd))


# 制作ldap自签名证书
def ssl_integration():
    conf_ldap_database()
    create_keepalived_file()
    # 判断证书目录是否存在，不存在则创建
    if not os.path.exists('/etc/openldap/certs'):
        os.mkdir("/etc/openldap/certs")
    # 生成my-ssl.conf文件
    my_ssl_conf = "/etc/openldap/certs/my-ssl.conf"
    text_my_ssl = '''authorityKeyIdentifier=keyid,issuer
basicConstraints=CA:TRUE
keyUsage = digitalSignature, nonRepudiation, keyEncipherment, dataEncipherment
extendedKeyUsage = serverAuth, clientAuth
subjectAltName = @alt_names
[ alt_names ]
DNS.1 = %s
IP.1 = %s''' % (ldap_domain_name, ldap_virtual_ip)
    with open(my_ssl_conf, 'w') as f:
        f.write(text_my_ssl)
    # 执行shell命令生成证书
    generate_ssl_certificate()

    # 修改证书所在目录权限
    subprocess.call("chown -R ldap:ldap /etc/openldap/certs".split(), shell=False)

    # 修改配置文件
    config_ldif = "/etc/openldap/slapd.d/cn=config.ldif"
    text_config_lidf = '''olcTLSCACertificateFile: /etc/openldap/certs/ca.crt
olcTLSCertificateFile: /etc/openldap/certs/ldap.crt
olcTLSCertificateKeyFile: /etc/openldap/certs/ldap.key
olcAllows: bind_v2'''
    with open(config_ldif, 'a') as f:
        f.write(text_config_lidf)
    # 修改ldap服务启动方式
    if "CentOS" in OS_TYPE or "Red Hat Enterprise Linux" in OS_TYPE:
        subprocess.call(['sed', '-i', '/^SLAPD_URLS=/c SLAPD_URLS="ldapi:/// ldap:/// ldaps:///"',
            '/etc/sysconfig/slapd'], shell=False)
    elif "Kylin" in OS_TYPE or "openEuler" in OS_TYPE or "KylinSec" in OS_TYPE:
        subprocess.call(['sed', '-i', '/^ExecStart=/c ExecStart=slapd -u ldap -h "ldaps:/// ldap:/// ldapi:///"',
            '/usr/lib/systemd/system/slapd.service'], shell=False)
    # 重启ldap服务器服务
    returncode = subprocess.call(["systemctl", "daemon-reload"], shell=False)
    if returncode == 0:
        subprocess.call(["systemctl", "restart", "slapd.service"], shell=False)
    # 查看ldap默认侦听端口是否为636
    ldap_port = subprocess.check_output(["netstat", "-antup"]).decode().strip()
    if "636" not in ldap_port:
        print("Failed to integrate TSL/SSL into the service order.")
        exit(255)
    # 关闭匿名认证
    with open("/etc/openldap/disable_anon.ldif", "w") as f:
        f.write(disable_anon_text)
    result_disable_anon = subprocess.call(
        ["ldapmodify", "-Y", "EXTERNAL", "-H", "ldapi:///", "-f", "/etc/openldap/disable_anon.ldif"], shell=False)
    if result_disable_anon != 0:
        print('Failed to disable LDAP anonymous authentication.')
        exit(255)
    # 配置安全端口
    if "Kylin" in OS_TYPE or "openEuler" in OS_TYPE or "KylinSec" in OS_TYPE:
        text_sync_ha = openEuler_text_sync_ha_sec
    elif "CentOS" in OS_TYPE or "Red Hat Enterprise Linux" in OS_TYPE:
        text_sync_ha = redhat_text_sync_ha_sec
    with open("/etc/openldap/sync-ha.ldif", 'w') as f:
        f.write(text_sync_ha)
    result_sync_ha = subprocess.call(
        ["ldapmodify", "-Y", "EXTERNAL", "-H", "ldapi:///", "-f", "/etc/openldap/sync-ha.ldif"], shell=False)
    if result_sync_ha != 0:
        print('Failed to import sync-ha.ldif to the database.')
        exit(255)
    subprocess.call(["systemctl", "restart", "slapd.service"], shell=False)
    ldap_status = subprocess.check_output(['systemctl', 'status', 'slapd'], universal_newlines=True)
    if "Active: active (running)" not in ldap_status:
        print("Slapd service is running abnormally.")
        exit(255)
    else:
        print('Ldap Master Configuration Completed')


ssl_integration()
